local SPELL_BONUS_SCHOOL_FIRE = 3
local SPELL_BONUS_SCHOOL_FROST = 5
local SPELL_BONUS_SCHOOL_ARCANE = 7

local SINGLE_TARGET_COEFFIENT = 0.95
local MULTI_TARGET_COEFFIENT = 0.33333

local frostDamageTalentMod = 1
local allDamageTalentMod = 1
local fireDamageTalentMod = 1

local SPELLS = {
    FROSTBOLT10 = {
        ID = 10181,
        BASE_DAMAGE = {
            MIN = 440,
            MAX = 475
        },
        CAST_TIME = 3,
        SCHOOL = SPELL_BONUS_SCHOOL_FROST,
        COEFFIENT = SINGLE_TARGET_COEFFIENT
    },
    CONE_OF_COLD5 = {
        ID = 10161,
        BASE_DAMAGE = {
            MIN = 338,
            MAX = 368
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FROST,
        COEFFIENT = MULTI_TARGET_COEFFIENT
    },
    FROST_NOVA4 = {
        ID = 10230,
        BASE_DAMAGE = {
            MIN = 73,
            MAX = 82
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FROST,
        COEFFIENT = 0.0714
    },
    BLIZZARD6 = {
        ID = 10187,
        BASE_DAMAGE = {
            MIN = 1192,
            MAX = 1192
        },
        CAST_TIME = 3.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FROST,
        COEFFIENT = MULTI_TARGET_COEFFIENT
    },
    FIREBALL11 = {
        ID = 10151,
        BASE_DAMAGE = {
            MIN = 561,
            MAX = 715
        },
        CAST_TIME = 3.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FIRE,
        COEFFIENT = SINGLE_TARGET_COEFFIENT
    },
    FIREBLAST7 = {
        ID = 10199,
        BASE_DAMAGE = {
            MIN = 446,
            MAX = 524
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FIRE,
        COEFFIENT = SINGLE_TARGET_COEFFIENT
    },
    SCORCH7 = {
        ID = 10207,
        BASE_DAMAGE = {
            MIN = 237,
            MAX = 280
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FIRE,
        COEFFIENT = SINGLE_TARGET_COEFFIENT
    },
    FLAMESTIKE6 = {
        ID = 10216,
        BASE_DAMAGE = {
            MIN = 381,
            MAX = 466
        },
        CAST_TIME = 3,
        SCHOOL = SPELL_BONUS_SCHOOL_ARCANE,
        COEFFIENT = 0.1761
    },
    PYROBLAST8 = {
        ID = 18809,
        BASE_DAMAGE = {
            MIN = 716,
            MAX = 890
        },
        CAST_TIME = 3.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FIRE,
        COEFFIENT = 1
    },
    BLAST_WAVE5 = {
        ID = 13021,
        BASE_DAMAGE = {
            MIN = 462,
            MAX = 544
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_FIRE,
        COEFFIENT = MULTI_TARGET_COEFFIENT * 0.95
    },
    ARCANE_EXPLOSION6 = {
        ID = 10202,
        BASE_DAMAGE = {
            MIN = 249,
            MAX = 270
        },
        CAST_TIME = 1.5,
        SCHOOL = SPELL_BONUS_SCHOOL_ARCANE,
        COEFFIENT = MULTI_TARGET_COEFFIENT
    },
    ARCANE_MISSLES7 = {
        ID = 10212,
        BASE_DAMAGE = {
            MIN = 196 * 5,
            MAX = 196 * 5
        },
        CAST_TIME = 3.5,
        SCHOOL = SPELL_BONUS_SCHOOL_ARCANE,
        COEFFIENT = 1.345
    }
}

local L = MageDamage.locale[GetLocale()] or MageDamage.locale.enUS;

-- TODO
-- Track buff Мощь тайной магии

local function checkTalents()
    local name, iconTexture, tier, column, rank, maxRank, isExceptional, available = GetTalentInfo(1, 15)
    allDamageTalentMod = 1 + rank * 0.01

    name, iconTexture, tier, column, rank, maxRank, isExceptional, available = GetTalentInfo(2, 15)
    fireDamageTalentMod = 1 + rank * 0.02

    name, iconTexture, tier, column, rank, maxRank, isExceptional, available = GetTalentInfo(3, 8)
    frostDamageTalentMod = 1 + rank * 0.02
end

GameTooltip:HookScript('OnTooltipSetSpell', function(tooltip)
    local spellName, spellID = tooltip:GetSpell()
    checkTalents()

    local minDamage
    local maxDamage

    local function calcDamage(spell)
        local talentMod = 1
        if (spell.SCHOOL == SPELL_BONUS_SCHOOL_FIRE) then
            talentMod = fireDamageTalentMod
        elseif (spell.SCHOOL == SPELL_BONUS_SCHOOL_FROST) then
            talentMod = frostDamageTalentMod
        end
        
        local spellBonusDamage = GetSpellBonusDamage(spell.SCHOOL) * spell.CAST_TIME / 3.5 * spell.COEFFIENT
        minDamage = math.floor((spell.BASE_DAMAGE.MIN + spellBonusDamage) * talentMod * allDamageTalentMod)
        maxDamage = math.ceil((spell.BASE_DAMAGE.MAX + spellBonusDamage) * talentMod * allDamageTalentMod)
    end

    if (spellID == SPELLS.FIREBALL11.ID) then
        calcDamage(SPELLS.FIREBALL11)
    elseif (spellID == SPELLS.FIREBLAST7.ID) then
        calcDamage(SPELLS.FIREBLAST7)
    elseif (spellID == SPELLS.SCORCH7.ID) then
        calcDamage(SPELLS.SCORCH7)
    elseif (spellID == SPELLS.FROSTBOLT10.ID) then
        calcDamage(SPELLS.FROSTBOLT10)
    elseif (spellID == SPELLS.ARCANE_EXPLOSION6.ID) then
        calcDamage(SPELLS.ARCANE_EXPLOSION6)
    elseif (spellID == SPELLS.CONE_OF_COLD5.ID) then
        calcDamage(SPELLS.CONE_OF_COLD5)
    elseif (spellID == SPELLS.FLAMESTIKE6.ID) then
        calcDamage(SPELLS.FLAMESTIKE6)
    elseif (spellID == SPELLS.FROST_NOVA4.ID) then
        calcDamage(SPELLS.FROST_NOVA4)
    elseif (spellID == SPELLS.BLIZZARD6.ID) then
        calcDamage(SPELLS.BLIZZARD6)
    elseif (spellID == SPELLS.ARCANE_MISSLES7.ID) then
        calcDamage(SPELLS.ARCANE_MISSLES7)
    elseif (spellID == SPELLS.PYROBLAST8.ID) then
        calcDamage(SPELLS.PYROBLAST8)
    elseif (spellID == SPELLS.BLAST_WAVE5.ID) then
        calcDamage(SPELLS.BLAST_WAVE5)
    end

    if (minDamage and maxDamage) then
        tooltip:AddLine(L.DAMAGE_TOTAL .. ' ' .. minDamage .. ' - ' .. maxDamage)
    -- For debug
    -- else
    --     tooltip:AddLine(spellID)
    end
end)
